'use strict';

var _path = require('path');

var _loaderUtils = require('loader-utils');

var _loaderUtils2 = _interopRequireDefault(_loaderUtils);

var _traverse = require('./traverse');

var _traverse2 = _interopRequireDefault(_traverse);

var _VirtualModulePlugin = require('./VirtualModulePlugin');

var _VirtualModulePlugin2 = _interopRequireDefault(_VirtualModulePlugin);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function collectStyles(src) {
  var tagName = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'css';

  var styles = [];

  // quick regex as an optimization to avoid parsing each file
  if (!src.match(new RegExp(tagName + '`([\\s\\S]*?)`', 'gmi'))) {
    return styles;
  }

  (0, _traverse2.default)(src, {
    TaggedTemplateExpression: function TaggedTemplateExpression(path) {
      var node = path.node;

      if (node.tag.name !== tagName || !path.scope.hasGlobal(tagName)) {
        return;
      }

      var parseError = path.buildCodeFrameError('Could not evaluate css. inline css must be statically analyzable');

      var start = node.start;
      var end = node.end;

      // remove the tag and evaluate as a plain template;

      path.replaceWith(node.quasi);

      var _path$evaluate = path.evaluate();

      var confident = _path$evaluate.confident;
      var value = _path$evaluate.value;


      if (!confident) {
        throw parseError;
      }

      styles.push({ value: value, start: start, end: end });
    }
  });

  return styles;
}

function replaceStyleTemplates(src, styles) {
  var offset = 0;

  function splice(str, start, end, replace) {
    var result = str.slice(0, start + offset) + replace + str.slice(end + offset);

    offset += replace.length - (end - start);
    return result;
  }

  styles.forEach(function (_ref) {
    var start = _ref.start;
    var end = _ref.end;
    var path = _ref.path;

    src = splice(src, start, end, 'require(\'' + path + '\')');
  });

  return src;
}

var LOADER_PLUGIN = Symbol('loader added VM plugin');

module.exports = function loader(content) {
  if (this.cacheable) this.cacheable();

  var query = _loaderUtils2.default.parseQuery(this.query);
  var styles = collectStyles(content, query.tagName);

  if (!styles.length) return content;

  var basepath = (0, _path.join)((0, _path.dirname)(this.resource), (0, _path.basename)(this.resource, (0, _path.extname)(this.resource)));
  var compilation = this._compilation; // eslint-disable-line no-underscore-dangle
  var plugin = compilation[LOADER_PLUGIN];

  if (!plugin) {
    plugin = compilation[LOADER_PLUGIN] = _VirtualModulePlugin2.default.bootstrap(compilation);
  }

  styles.forEach(function (style, idx) {
    style.path = basepath + '__extracted_styles_' + idx++ + '.css';
    plugin.addFile(style.path, style.value);
  });

  return replaceStyleTemplates(content, styles);
};